@@ -2,7 +2,7 @@
 
   digest -- hash digest functions for R
 
-  Copyright (C) 2003 - 2015  Dirk Eddelbuettel <edd@debian.org>
+  Copyright (C) 2003 - 2016  Dirk Eddelbuettel <edd@debian.org>
 
   This file is part of digest.
 
@@ -27,6 +27,7 @@
 #include <R.h>
 #include <Rdefines.h>
 #include <Rinternals.h>
+#include <Rversion.h>
 #include <inttypes.h>
 #include "sha1.h"
 #include "sha2.h"
@@ -58,14 +59,18 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
     int output_length = -1;
     if (IS_RAW(Txt)) { /* Txt is either RAW */
         txt = (char*) RAW(Txt);
+#if defined(R_VERSION) && R_VERSION >= R_Version(3,0,0)        
         nChar = XLENGTH(Txt);
+#else        
+        nChar = LENGTH(Txt);
+#endif
     } else { /* or a string */
         txt = (char*) STRING_VALUE(Txt);
         nChar = strlen(txt);
     }
-    if (skip>0) {
+    if (skip > 0 && algo < 100) {
         if (skip>=nChar) {
-            nChar=0;
+            nChar=0;                                                            /* #nocov */ 
         } else {
             nChar -= skip;
             txt += skip;
@@ -114,8 +119,7 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
         val  = digest_crc32(0L, 0, 0);
         val  = digest_crc32(val, (unsigned char*) txt, (unsigned) l);
 
-        sprintf(output, "%2.2x", (unsigned int) val);
-
+        sprintf(output, "%08x", (unsigned int) val);
         break;
     }
     case 4: {     /* sha256 case */
@@ -192,7 +196,7 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
         unsigned char md5sum[16];
 
         if (!(fp = fopen(txt,"rb"))) {
-            error("Cannot open input file: %s", txt); /* already covered at R level too */
+            error("Cannot open input file: %s", txt); /* already covered at R level too */ /* #nocov */ 
         }
         if (skip > 0) fseek(fp, skip, SEEK_SET);
         md5_starts( &ctx );
@@ -223,7 +227,7 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
         unsigned char sha1sum[20];
 
         if (!(fp = fopen(txt,"rb"))) {
-            error("Cannot open input file: %s", txt); /* already covered at R level too */
+            error("Cannot open input file: %s", txt); /* already covered at R level too */ /* #nocov */ 
         }
         if (skip > 0) fseek(fp, skip, SEEK_SET);
         sha1_starts ( &ctx );
@@ -250,7 +254,7 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
         unsigned long val;
 
         if (!(fp = fopen(txt,"rb"))) {
-            error("Cannot open input file: %s", txt); /* already covered at R level too */
+            error("Cannot open input file: %s", txt); /* already covered at R level too */ /* #nocov */ 
         }
         if (skip > 0) fseek(fp, skip, SEEK_SET);
         val  = digest_crc32(0L, 0, 0);
@@ -265,7 +269,7 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
                 val  = digest_crc32(val , buf, (unsigned) nChar);
         }
         fclose(fp);
-        sprintf(output, "%2.2x", (unsigned int) val);
+        sprintf(output, "%08x", (unsigned int) val);
         break;
     }
     case 104: {     /* sha256 file case */
@@ -276,7 +280,7 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
         unsigned char sha256sum[32];
 
         if (!(fp = fopen(txt,"rb"))) {
-            error("Cannot open input file: %s", txt); /* already covered at R level too */
+            error("Cannot open input file: %s", txt); /* already covered at R level too */ /* #nocov */ 
         }
         if (skip > 0) fseek(fp, skip, SEEK_SET);
         sha256_starts ( &ctx );
@@ -307,7 +311,7 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
         unsigned char buf[BUF_SIZE];
 
         if (!(fp = fopen(txt,"rb"))) {
-            error("Cannot open input file: %s", txt); /* already covered at R level too */
+            error("Cannot open input file: %s", txt); /* already covered at R level too */ /* #nocov */ 
         }
         if (skip > 0) fseek(fp, skip, SEEK_SET);
         SHA512_Init(&ctx);
@@ -345,7 +349,7 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
         XXH32_state_t state;
 
         if (!(fp = fopen(txt,"rb"))) {
-            error("Cannot open input file: %s", txt); /* already covered at R level too */
+            error("Cannot open input file: %s", txt); /* already covered at R level too */ /* #nocov */ 
         }
         if (skip > 0) fseek(fp, skip, SEEK_SET);
         XXH32_reset(&state, seed);
@@ -370,7 +374,7 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
         XXH64_state_t state;
 
         if (!(fp = fopen(txt,"rb"))) {
-            error("Cannot open input file: %s", txt); /* already covered at R level too */
+            error("Cannot open input file: %s", txt); /* already covered at R level too */ /* #nocov */ 
         }
         if (skip > 0) fseek(fp, skip, SEEK_SET);
         XXH64_reset(&state, seed);
@@ -401,7 +405,7 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
         size_t total_length = 0;
 
         if (!(fp = fopen(txt,"rb"))) {
-            error("Cannot open input file: %s", txt); /* already covered at R level too */
+            error("Cannot open input file: %s", txt); /* already covered at R level too */ /* #nocov */ 
         }
         if (skip > 0) fseek(fp, skip, SEEK_SET);
         XXH64_reset(&state, seed);
@@ -426,7 +430,7 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw, SEXP Se
         break;
     }
     default: {
-        error("Unsupported algorithm code"); /* should not be reached due to test in R */
+        error("Unsupported algorithm code"); /* should not be reached due to test in R */ /* #nocov */ 
     }
     } /* end switch */
 
